@@ -48,6 +48,7 @@
 
 enum {
 	SELECTED_DEVICE_CHANGED,
+	SELECTED_DEVICE_ACTIVATED,
 	LAST_SIGNAL
 };
 
@@ -62,14 +63,14 @@ struct _BluetoothChooserPrivate {
 	BluetoothClient *client;
 	GtkTreeSelection *selection;
 	GtkTreeModel *model, *filter, *adapter_model;
-	GtkWidget *label;
+	GtkWidget *label, *label_align;
 
 	gulong default_adapter_changed_id;
 
 	/* Widgets/UI bits that can be shown or hidden */
 	GtkCellRenderer *bonded_cell;
 	GtkCellRenderer *connected_cell;
-	GtkWidget *treeview;
+	GtkWidget *treeview, *scrolled_window;
 	GtkWidget *search_hbox, *search_label, *spinner;
 	GtkWidget *filters_vbox;
 
@@ -482,6 +483,22 @@ bluetooth_chooser_get_treeview (BluetoothChooser *self)
 	return priv->treeview;
 }
 
+/**
+ * bluetooth_chooser_get_scrolled_window:
+ * @self: A #BluetoothChooser widget.
+ *
+ * Returns the #GtkScrolledWindow object for the #BluetoothChooser.
+ *
+ * Return value: a #GtkScrolledWindow object.
+ **/
+GtkWidget *
+bluetooth_chooser_get_scrolled_window (BluetoothChooser *self)
+{
+	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(self);
+
+	return priv->scrolled_window;
+}
+
 /**
  * bluetooth_chooser_set_title:
  * @self: a BluetoothChooser widget.
@@ -495,6 +512,7 @@ bluetooth_chooser_set_title (BluetoothChooser  *self, const char *title)
 	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(self);
 
 	if (title == NULL) {
+		gtk_alignment_set_padding (GTK_ALIGNMENT (priv->label_align), 0, 0, 0, 0);
 		gtk_widget_hide (priv->label);
 	} else {
 		char *str;
@@ -503,6 +521,7 @@ bluetooth_chooser_set_title (BluetoothChooser  *self, const char *title)
 		gtk_label_set_markup (GTK_LABEL(priv->label), str);
 		g_free (str);
 		gtk_widget_show (priv->label);
+		gtk_alignment_set_padding (GTK_ALIGNMENT (priv->label_align), 0, 0, 12, 0);
 	}
 }
 
@@ -542,6 +561,21 @@ select_browse_device_callback (GtkTreeSelection *selection, gpointer user_data)
 	g_free (address);
 }
 
+static void
+row_activated_cb (GtkTreeView       *tree_view,
+		  GtkTreePath       *path,
+		  GtkTreeViewColumn *column,
+		  BluetoothChooser  *self)
+{
+	char *address;
+
+	address = bluetooth_chooser_get_selected_device (self);
+	g_signal_emit (G_OBJECT (self),
+		       selection_table_signals[SELECTED_DEVICE_ACTIVATED],
+		       0, address);
+	g_free (address);
+}
+
 static gboolean
 filter_type_func (GtkTreeModel *model, GtkTreeIter *iter, BluetoothChooserPrivate *priv)
 {
@@ -741,9 +775,13 @@ create_treeview (BluetoothChooser *self)
 
 	g_object_set (tree, "show-expanders", FALSE, NULL);
 
+	g_signal_connect (G_OBJECT (tree), "row-activated",
+			  G_CALLBACK (row_activated_cb), self);
+
 	column = gtk_tree_view_column_new ();
 
 	gtk_tree_view_column_set_title (column, _("Device"));
+	gtk_tree_view_column_set_expand (GTK_TREE_VIEW_COLUMN(column), TRUE);
 
 	/* The type icon */
 	renderer = gtk_cell_renderer_pixbuf_new ();
@@ -777,8 +815,6 @@ create_treeview (BluetoothChooser *self)
 
 	gtk_tree_view_append_column (GTK_TREE_VIEW(tree), column);
 
-	gtk_tree_view_column_set_min_width (GTK_TREE_VIEW_COLUMN(column), 280);
-
 	gtk_tree_view_insert_column_with_data_func (GTK_TREE_VIEW(tree), -1,
 						    _("Type"), gtk_cell_renderer_text_new(),
 						    type_to_text, NULL, NULL);
@@ -817,9 +853,7 @@ bluetooth_chooser_init(BluetoothChooser *self)
 	BluetoothChooserPrivate *priv = BLUETOOTH_CHOOSER_GET_PRIVATE(self);
 
 	GtkWidget *vbox;
-	GtkWidget *alignment;
 	GtkWidget *hbox;
-	GtkWidget *scrolled_window;
 
 	gtk_widget_push_composite_child ();
 
@@ -840,19 +874,22 @@ bluetooth_chooser_init(BluetoothChooser *self)
 	gtk_label_set_use_markup (GTK_LABEL (priv->label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (priv->label), 0, 0.5);
 
-	alignment = gtk_alignment_new (0.5, 0.5, 1, 1);
-	gtk_widget_show (alignment);
-	gtk_box_pack_start (GTK_BOX (vbox), alignment, TRUE, TRUE, 0);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 12, 0);
+	priv->label_align = gtk_alignment_new (0.5, 0.5, 1, 1);
+	gtk_widget_show (priv->label_align);
+	gtk_box_pack_start (GTK_BOX (vbox), priv->label_align, TRUE, TRUE, 0);
+	gtk_alignment_set_padding (GTK_ALIGNMENT (priv->label_align), 0, 0, 0, 0);
 
 	/* The treeview label */
 	vbox = gtk_vbox_new (FALSE, 6);
 	gtk_widget_show (vbox);
-	gtk_container_add (GTK_CONTAINER (alignment), vbox);
+	gtk_container_add (GTK_CONTAINER (priv->label_align), vbox);
 
-	hbox = gtk_hbox_new (FALSE, 24);
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
+	priv->search_hbox = gtk_hbox_new (FALSE, 24);
+	gtk_widget_set_name (priv->search_hbox, "search_hbox");
+	if (priv->show_searching)
+		gtk_widget_show (priv->search_hbox);
+	gtk_box_pack_start (GTK_BOX (vbox), priv->search_hbox, FALSE, TRUE, 0);
+	gtk_widget_set_no_show_all (priv->search_hbox, TRUE);
 
 	/* Setup the adapter disco mode callback for the search button */
 	priv->adapter_model = bluetooth_client_get_adapter_model (priv->client);
@@ -860,26 +897,25 @@ bluetooth_chooser_init(BluetoothChooser *self)
 			  G_CALLBACK (adapter_model_row_changed), self);
 
 	/* The searching label */
-	priv->search_hbox = gtk_hbox_new (FALSE, 6);
-	gtk_widget_set_no_show_all (priv->search_hbox, TRUE);
+	hbox = gtk_hbox_new (FALSE, 6);
+	gtk_widget_set_name (hbox, "searching label hbox");
 	priv->spinner = gtk_spinner_new ();
-	gtk_container_add (GTK_CONTAINER (priv->search_hbox), priv->spinner);
+	gtk_container_add (GTK_CONTAINER (hbox), priv->spinner);
 	gtk_widget_show (priv->spinner);
 	priv->search_label = gtk_label_new (_("Searching for devices..."));
-	gtk_container_add (GTK_CONTAINER (priv->search_hbox), priv->search_label);
+	gtk_container_add (GTK_CONTAINER (hbox), priv->search_label);
 	gtk_widget_show (priv->search_label);
+	gtk_widget_show (hbox);
 
-	gtk_box_pack_end (GTK_BOX (hbox), priv->search_hbox, FALSE, TRUE, 0);
-	if (priv->show_searching)
-		gtk_widget_show (priv->search_hbox);
+	gtk_box_pack_end (GTK_BOX (priv->search_hbox), hbox, FALSE, TRUE, 0);
 	//FIXME check whether the default adapter is discovering right now
 
 	/* The treeview */
-	scrolled_window = create_treeview (self);
-	gtk_widget_show_all (scrolled_window);
-	gtk_box_pack_start (GTK_BOX (vbox), scrolled_window, TRUE, TRUE, 0);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window), GTK_SHADOW_IN);
+	priv->scrolled_window = create_treeview (self);
+	gtk_widget_show_all (priv->scrolled_window);
+	gtk_box_pack_start (GTK_BOX (vbox), priv->scrolled_window, TRUE, TRUE, 0);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (priv->scrolled_window), GTK_SHADOW_IN);
 
 	/* The filters */
 	priv->device_type_filter_model = GTK_TREE_MODEL (gtk_list_store_new (DEVICE_TYPE_FILTER_NUM_COLS,
@@ -1146,6 +1182,22 @@ bluetooth_chooser_class_init (BluetoothChooserClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1, G_TYPE_STRING);
+	/**
+	 * BluetoothChooser::selected-device-activated:
+	 * @chooser: a #BluetoothChooser widget which received the signal
+	 * @address: the Bluetooth address for the currently selected device, or %NULL
+	 *
+	 * The #BluetoothChooser:selected-device-activated signal is launched when a
+	 * device is double-clicked in the chooser.
+	 **/
+	selection_table_signals[SELECTED_DEVICE_ACTIVATED] =
+		g_signal_new ("selected-device-activated",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (BluetoothChooserClass, selected_device_activated),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__STRING,
+			      G_TYPE_NONE, 1, G_TYPE_STRING);
 
 	g_object_class_install_property (G_OBJECT_CLASS(klass),
 					 PROP_TITLE, g_param_spec_string ("title",
