@@ -1664,6 +1664,36 @@ bluetooth_client_get_discoverable (BluetoothClient *client)
 	return ret;
 }
 
+/**
+ * bluetooth_client_get_name:
+ * @client: a #BluetoothClient
+ *
+ * Gets the default adapter's name, cached in the adapter model.
+ *
+ * Returns: the name of the adapter, or %NULL if no default adapter exists
+ */
+char *
+bluetooth_client_get_name (BluetoothClient *client)
+{
+	BluetoothClientPrivate *priv;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	char *ret;
+
+	g_return_val_if_fail (BLUETOOTH_IS_CLIENT (client), FALSE);
+
+	priv = BLUETOOTH_CLIENT_GET_PRIVATE (client);
+	if (priv->default_adapter == NULL)
+		return NULL;
+
+	path = gtk_tree_row_reference_get_path (priv->default_adapter);
+	gtk_tree_model_get_iter (GTK_TREE_MODEL (priv->store), &iter, path);
+	gtk_tree_model_get (GTK_TREE_MODEL (priv->store), &iter,
+                            BLUETOOTH_COLUMN_NAME, &ret, -1);
+
+	return ret;
+}
+
 /**
  * bluetooth_client_set_discoverable:
  * @client: a #BluetoothClient object
@@ -2141,13 +2171,21 @@ bluetooth_client_dump_device (GtkTreeModel *model,
 			    BLUETOOTH_COLUMN_UUIDS, &uuids,
 			    BLUETOOTH_COLUMN_PROXY, &proxy,
 			    -1);
-	is_adapter = !gtk_tree_model_iter_parent (model, &parent, iter);
+	if (proxy) {
+		char *basename;
+		basename = g_path_get_basename(dbus_g_proxy_get_path(proxy));
+		is_adapter = !g_str_has_prefix (basename, "dev_");
+		g_free (basename);
+	} else {
+		is_adapter = !gtk_tree_model_iter_parent (model, &parent, iter);
+	}
 
 	if (is_adapter != FALSE) {
 		/* Adapter */
 		g_print ("Adapter: %s (%s)\n", name, address);
 		if (is_default)
 			g_print ("\tDefault adapter\n");
+		g_print ("\tD-Bus Path: %s\n", proxy ? dbus_g_proxy_get_path (proxy) : "(none)");
 		g_print ("\tDiscoverable: %s\n", BOOL_STR (discoverable));
 		if (discovering)
 			g_print ("\tDiscovery in progress\n");
