@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *
  *  BlueZ - Bluetooth protocol stack for Linux
@@ -32,13 +33,13 @@
 #include <glib/gi18n.h>
 #include <math.h>
 #include <gdk/gdkkeysyms.h>
-#include <mx/mx-gtk.h>
+#include <mx-gtk/mx-gtk.h>
 
 #include "bluetooth-client.h"
 #include "bluetooth-client-private.h"
 #include "bluetooth-chooser.h"
 #include "bluetooth-chooser-private.h"
-#include "bluetooth-killswitch.h"
+#include "bluetooth-powerswitch.h"
 #include "bluetooth-plugin-manager.h"
 #include "bluetooth-filter-widget.h"
 #include "bluetooth-agent.h"
@@ -47,6 +48,7 @@
 #include "pin.h"
 
 #include "mux-cell-renderer-text.h"
+#include "mux-banner.h"
 #include "koto-cell-renderer-pixbuf.h"
 
 #include "moblin-panel.h"
@@ -59,19 +61,23 @@ G_DEFINE_TYPE (MoblinPanel, moblin_panel, GTK_TYPE_HBOX)
 struct _MoblinPanelPrivate
 {
 	/* Bluetooth objects for interacting with BlueZ */
-	BluetoothKillswitch *killswitch;
+	BluetoothPowerswitch *powerswitch;
 	BluetoothClient *client;
 	BluetoothAgent *agent;
 	GtkTreeModel *chooser_model;
 
 	/* Page widgets that need to be "globally" accessible */
 	GtkWidget *power_switch;
+	guint      visible_countdown; /* seconds */
+	GtkWidget *visible_label;
+	GtkWidget *visible_button;
 	GtkWidget *notebook;
 	GtkWidget *label_pin_help;
 	GtkWidget *label_pin;
 	GtkWidget *label_ssp_pin_help;
 	GtkWidget *label_ssp_pin;
 	GtkWidget *summary_frame;
+	GtkWidget *extra_config_label;
 	GtkWidget *extra_config_vbox;
 	GtkWidget *label_failure;
 	GtkWidget *chooser;
@@ -108,6 +114,7 @@ struct _MoblinPanelPrivate
 };
 
 #define CONNECT_TIMEOUT 3.0
+#define DISCOVER_TIMEOUT (2 * 60)
 #define AGENT_PATH "/org/bluez/agent/moblin"
 
 typedef struct {
@@ -148,12 +155,113 @@ power_switch_toggled_cb (MxGtkLightSwitch *light_switch,
 	MoblinPanelPrivate *priv = MOBLIN_PANEL_GET_PRIVATE(user_data);
 
 	if (state == TRUE) {
-		bluetooth_killswitch_set_state (priv->killswitch, KILLSWITCH_STATE_UNBLOCKED);
+		bluetooth_powerswitch_set_state (priv->powerswitch, POWERSWITCH_STATE_ON);
 	} else {
-		bluetooth_killswitch_set_state (priv->killswitch, KILLSWITCH_STATE_SOFT_BLOCKED);
+		bluetooth_powerswitch_set_state (priv->powerswitch, POWERSWITCH_STATE_OFF);
 	}
 }
 
+static char *
+totem_time_to_string_text (gint64 msecs)
+{
+	char *secs, *mins, *hours, *string;
+	int sec, min, hour, _time;
+
+	_time = (int) (msecs / 1000);
+	sec = _time % 60;
+	_time = _time - sec;
+	min = (_time % (60*60)) / 60;
+	_time = _time - (min * 60);
+	hour = _time / (60*60);
+
+	hours = g_strdup_printf (ngettext ("%d hour", "%d hours", hour), hour);
+
+	mins = g_strdup_printf (ngettext ("%d minute",
+					  "%d minutes", min), min);
+
+	secs = g_strdup_printf (ngettext ("%d second",
+					  "%d seconds", sec), sec);
+
+	if (hour > 0)
+	{
+		/* hour:minutes:seconds */
+		string = g_strdup_printf (_("%s %s %s"), hours, mins, secs);
+	} else if (min > 0) {
+		if (sec) {
+			/* minutes:seconds */
+			string = g_strdup_printf (_("%s %s"), mins, secs);
+		} else {
+			/* minutes */
+			string = g_strdup_printf (_("%s"), mins);
+		}
+	} else if (sec > 0) {
+		/* seconds */
+		string = g_strdup_printf (_("%s"), secs);
+	} else {
+		/* 0 seconds */
+		string = g_strdup (_("0 seconds"));
+	}
+
+	g_free (hours);
+	g_free (mins);
+	g_free (secs);
+
+	return string;
+}
+
+static gboolean
+update_visible (gpointer data)
+{
+	MoblinPanel *panel = MOBLIN_PANEL (data);
+	MoblinPanelPrivate *priv = MOBLIN_PANEL_GET_PRIVATE (panel);
+	char *label, *time;
+
+	if (priv->visible_countdown > 0) {
+		gtk_widget_hide (priv->visible_button);
+		gtk_widget_show (priv->visible_label);
+
+		time = totem_time_to_string_text (priv->visible_countdown * 1000);
+		label = g_strdup_printf (_("Your computer is visible on\n"
+					   "Bluetooth for %s."), time);
+		gtk_label_set_text (GTK_LABEL (priv->visible_label), label);
+		g_free (label);
+		g_free (time);
+
+		priv->visible_countdown -= 5;
+
+		return TRUE;
+	} else {
+		bluetooth_client_set_discoverable (priv->client, FALSE, 0);
+
+		gtk_widget_show (priv->visible_button);
+		gtk_widget_hide (priv->visible_label);
+
+		return FALSE;
+	}
+}
+
+static void
+make_discoverable (MoblinPanel *panel)
+{
+	MoblinPanelPrivate *priv = MOBLIN_PANEL_GET_PRIVATE (panel);
+
+	if (bluetooth_powerswitch_get_state (priv->powerswitch) != POWERSWITCH_STATE_ON)
+		return;
+
+	bluetooth_client_set_discoverable (priv->client, TRUE, DISCOVER_TIMEOUT);
+
+	priv->visible_countdown = DISCOVER_TIMEOUT;
+
+	update_visible (panel);
+	g_timeout_add_seconds (5, update_visible, panel);
+}
+
+static void
+visible_button_cb (GtkButton *button, gpointer user_data)
+{
+	make_discoverable (MOBLIN_PANEL (user_data));
+}
+
 static void
 enable_send_file (MoblinPanel *self)
 {
@@ -185,8 +293,8 @@ enable_send_file (MoblinPanel *self)
 }
 
 static void
-killswitch_state_changed_cb (BluetoothKillswitch *killswitch,
-                             KillswitchState      state,
+powerswitch_state_changed_cb (BluetoothPowerswitch *powerswitch,
+                             PowerswitchState      state,
                              gpointer             user_data)
 {
 	MoblinPanel *self = MOBLIN_PANEL (user_data);
@@ -194,19 +302,22 @@ killswitch_state_changed_cb (BluetoothKillswitch *killswitch,
 
 	g_signal_handlers_block_by_func (priv->power_switch, power_switch_toggled_cb, user_data);
 
-	if (state == KILLSWITCH_STATE_SOFT_BLOCKED) {
+	if (state == POWERSWITCH_STATE_OFF) {
 		mx_gtk_light_switch_set_active (MX_GTK_LIGHT_SWITCH (priv->power_switch),
 	                                      FALSE);
 		gtk_widget_set_sensitive (priv->power_switch, TRUE);
+		gtk_widget_set_sensitive (priv->visible_button, FALSE);
 		gtk_widget_set_sensitive (priv->add_new_button, FALSE);
 		gtk_widget_set_sensitive (priv->send_button, FALSE);
-	} else if (state == KILLSWITCH_STATE_UNBLOCKED) {
+	} else if (state == POWERSWITCH_STATE_ON) {
 		mx_gtk_light_switch_set_active (MX_GTK_LIGHT_SWITCH (priv->power_switch), TRUE);
 		gtk_widget_set_sensitive (priv->power_switch, TRUE);
+		gtk_widget_set_sensitive (priv->visible_button, TRUE);
 		gtk_widget_set_sensitive (priv->add_new_button, TRUE);
 		enable_send_file (self);
-	} else if (state == KILLSWITCH_STATE_HARD_BLOCKED || state == KILLSWITCH_STATE_NO_ADAPTER) {
+	} else if (state == POWERSWITCH_STATE_NO_ADAPTER) {
 		gtk_widget_set_sensitive (priv->power_switch, FALSE);
+		gtk_widget_set_sensitive (priv->visible_button, FALSE);
 		gtk_widget_set_sensitive (priv->add_new_button, FALSE);
 		gtk_widget_set_sensitive (priv->send_button, FALSE);
 	} else {
@@ -288,8 +399,8 @@ entry_custom_event (GtkWidget *entry, GdkEventKey *event)
 	if (event->length == 0)
 		return FALSE;
 
-	if ((event->keyval >= GDK_0 && event->keyval <= GDK_9) ||
-	    (event->keyval >= GDK_KP_0 && event->keyval <= GDK_KP_9))
+	if ((event->keyval >= GDK_KEY_0 && event->keyval <= GDK_KEY_9) ||
+	    (event->keyval >= GDK_KEY_KP_0 && event->keyval <= GDK_KEY_KP_9))
 		return FALSE;
 
 	return TRUE;
@@ -323,42 +434,27 @@ static void
 set_user_pincode (GtkWidget *button, gpointer user_data)
 {
 	MoblinPanelPrivate *priv = MOBLIN_PANEL_GET_PRIVATE (user_data);
-	GSList *list, *l;
+	GtkEntry *entry;
+	const char *pin;
 
-	list = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
-	for (l = list; l ; l = l->next) {
-		GtkEntry *entry;
-		GtkWidget *radio;
-		const char *pin;
+	if (! gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
+		return;
 
-		if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
-			continue;
+	pin = g_object_get_data (G_OBJECT (button), "pin");
+	entry = g_object_get_data (G_OBJECT (button), "entry");
 
-		/* Is it radio_fixed that changed? */
-		radio = g_object_get_data (G_OBJECT (button), "button");
-		if (radio != NULL) {
-			set_user_pincode (radio, user_data);
-			return;
-		}
-
-		pin = g_object_get_data (G_OBJECT (button), "pin");
-		entry = g_object_get_data (G_OBJECT (button), "entry");
-
-		if (entry != NULL) {
-			g_free (priv->user_pincode);
-			priv->user_pincode = g_strdup (gtk_entry_get_text(entry));
-			gtk_dialog_set_response_sensitive (GTK_DIALOG (priv->pin_dialog),
-							   GTK_RESPONSE_ACCEPT,
-							   gtk_entry_get_text_length (entry) >= 1);
-		} else if (pin != NULL) {
-			g_free (priv->user_pincode);
-			if (*pin == '\0')
-				priv->user_pincode = NULL;
-			else
-				priv->user_pincode = g_strdup (pin);
-		}
-
-		break;
+	if (entry != NULL) {
+		g_free (priv->user_pincode);
+		priv->user_pincode = g_strdup (gtk_entry_get_text(entry));
+		gtk_dialog_set_response_sensitive (GTK_DIALOG (priv->pin_dialog),
+						   GTK_RESPONSE_ACCEPT,
+						   gtk_entry_get_text_length (entry) >= 1);
+	} else if (pin != NULL) {
+		g_free (priv->user_pincode);
+		if (*pin == '\0')
+			priv->user_pincode = NULL;
+		else
+			priv->user_pincode = g_strdup (pin);
 	}
 }
 
@@ -404,31 +500,27 @@ pin_options_button_clicked_cb (GtkButton *button,
 		g_object_set_data (G_OBJECT (priv->radio_1111), "pin", "1111");
 		g_object_set_data (G_OBJECT (priv->radio_1234), "pin", "1234");
 		g_object_set_data (G_OBJECT (priv->radio_custom), "entry", priv->entry_custom);
-
-		gtk_dialog_add_button (GTK_DIALOG (priv->pin_dialog), GTK_STOCK_OK, GTK_RESPONSE_ACCEPT);
 	}
 
-	oldpin = priv->user_pincode;
-	priv->user_pincode = NULL;
+	oldpin = g_strdup (priv->user_pincode);
 
 	gtk_window_present (GTK_WINDOW (priv->pin_dialog));
 
-	if (oldpin == NULL)
+	if (priv->user_pincode == NULL) {
 		radio = priv->radio_auto;
-	else if (g_str_equal (oldpin, "0000"))
+	} else if (g_str_equal (priv->user_pincode, "0000")) {
 		radio = priv->radio_0000;
-	else if (g_str_equal (oldpin, "1111"))
+	} else if (g_str_equal (priv->user_pincode, "1111")) {
 		radio = priv->radio_1111;
-	else if (g_str_equal (oldpin, "1234"))
+	} else if (g_str_equal (priv->user_pincode, "1234")) {
 		radio = priv->radio_1234;
-	else
+	} else {
 		radio = priv->radio_custom;
-
+		gtk_entry_set_text (GTK_ENTRY (priv->entry_custom), priv->user_pincode);
+	}
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio), TRUE);
-	if (radio == priv->radio_custom)
-		gtk_entry_set_text (GTK_ENTRY (priv->entry_custom), oldpin);
 
-	if (gtk_dialog_run (GTK_DIALOG (priv->pin_dialog)) != GTK_RESPONSE_ACCEPT) {
+	if (gtk_dialog_run (GTK_DIALOG (priv->pin_dialog)) != GTK_RESPONSE_OK) {
 		g_free (priv->user_pincode);
 		priv->user_pincode = oldpin;
 	} else {
@@ -681,7 +773,6 @@ set_current_page (MoblinPanel *self, MoblinPages page)
 		const char **uuids;
 		GList *widgets;
 		char *title;
-		gboolean have_additional_widgets = FALSE;
 
 		title = g_strdup_printf (_("Successfully set up new device '%s'"), priv->target_name);
 		set_frame_title (GTK_FRAME (priv->summary_frame), title);
@@ -704,15 +795,14 @@ set_current_page (MoblinPanel *self, MoblinPages page)
 						    0);
 
 			g_list_free (widgets);
+			gtk_widget_show (priv->extra_config_label);
 			gtk_widget_show_all (priv->extra_config_vbox);
-			have_additional_widgets = TRUE;
+		} else {
+			gtk_widget_hide (priv->extra_config_label);
+			gtk_widget_hide (priv->extra_config_vbox);
 		}
 
 		g_value_unset (&value);
-
-		if (!have_additional_widgets)
-			/* Show the devices page if we don't have any extra information to present. */
-			page = PAGE_DEVICES;
 	}
 
 	if (page == PAGE_DEVICES) {
@@ -892,7 +982,9 @@ pair_to_text (GtkTreeViewColumn *column, GtkCellRenderer *cell,
 			BLUETOOTH_COLUMN_TRUSTED, &trusted, -1);
 
 	if (!paired && !trusted) {
-		g_object_set (cell, "markup", _("<u>Pair</u>"), NULL);
+		gchar *markup = g_strdup_printf ("<u>%s</u>", _("Pair"));
+		g_object_set (cell, "markup", markup, NULL);
+		g_free (markup);
 	}
 }
 
@@ -906,7 +998,9 @@ connect_to_text (GtkTreeViewColumn *column, GtkCellRenderer *cell,
 			BLUETOOTH_COLUMN_TRUSTED, &trusted, -1);
 
 	if ((paired || trusted) && connected == FALSE) {
-		g_object_set (cell, "markup", _("<u>Connect</u>"), NULL);
+		gchar *markup = g_strdup_printf ("<u>%s</u>", _("Connect"));
+		g_object_set (cell, "markup", markup, NULL);
+		g_free (markup);
 	} else {
 		g_object_set (cell, "markup", "", NULL);
 	}
@@ -925,7 +1019,10 @@ browse_to_text (GtkTreeViewColumn *column, GtkCellRenderer *cell,
 	if (uuids != NULL) {
 		for (i = 0; uuids[i] != NULL; i++)
 			if (g_str_equal (uuids[i], "OBEXFileTransfer")) {
-				g_object_set (cell, "markup", _("<u>Browse</u>"), NULL);
+				gchar *markup = g_strdup_printf ("<u>%s</u>", _("Browse"));
+				g_object_set (cell, "markup", markup, NULL);
+				g_free (markup);
+
 				found = TRUE;
 				break;
 			}
@@ -1160,10 +1257,10 @@ create_summary_page (MoblinPanel *self)
 	vbox = gtk_vbox_new (FALSE, 12);
 	gtk_widget_show (vbox);
 	gtk_container_add (GTK_CONTAINER (page), vbox);
-	w = gtk_label_new (_("Select the additional services you want to use with your device:"));
-	gtk_misc_set_alignment (GTK_MISC (w), 0.0, 0.5);
-	gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 6);
-	gtk_widget_show (w);
+	priv->extra_config_label = gtk_label_new (_("Select the additional services you want to use with your device:"));
+	gtk_misc_set_alignment (GTK_MISC (priv->extra_config_label), 0.0, 0.5);
+	gtk_widget_show (priv->extra_config_label);
+	gtk_box_pack_start (GTK_BOX (vbox), priv->extra_config_label, FALSE, FALSE, 6);
 
 	priv->extra_config_vbox = gtk_vbox_new (FALSE, 6);
 	gtk_widget_show (priv->extra_config_vbox);
@@ -1273,8 +1370,10 @@ create_add_page (MoblinPanel *self)
 	GtkWidget *page;
 	GtkWidget *vbox, *hbox;
 	GtkWidget *filter;
-	GtkWidget *frame_title;
+	GtkWidget *frame_vbox;
+	GtkWidget *banner;
 	GtkWidget *frame;
+	GtkWidget *alignment;
 	GtkWidget *back_button;
 	GtkWidget *pin_button;
 	GtkWidget *tree_view;
@@ -1283,18 +1382,34 @@ create_add_page (MoblinPanel *self)
 
 	priv = MOBLIN_PANEL_GET_PRIVATE (self);
 
-	page = gtk_hbox_new (FALSE, 0);
+	page = gtk_hbox_new (FALSE, 8);
+	gtk_container_set_border_width (GTK_CONTAINER (page), 8);
 	gtk_widget_show (page);
 
 	/* Add child widgetry */
 	vbox = gtk_vbox_new (FALSE, 4);
 	gtk_widget_show (vbox);
-	gtk_box_pack_start (GTK_BOX (page), vbox, TRUE, TRUE, 4);
+	gtk_box_pack_start (GTK_BOX (page), vbox, TRUE, TRUE, 0);
 
-	frame = mx_gtk_frame_new ();
-	frame_title = gtk_label_new ("");
-	gtk_frame_set_label_widget (GTK_FRAME (frame), frame_title);
-	set_frame_title (GTK_FRAME (frame), _("Devices"));
+	frame = gtk_frame_new (NULL);
+	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
+	gtk_widget_show (frame);
+	gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
+
+	frame_vbox = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (frame_vbox);
+	gtk_container_add (GTK_CONTAINER (frame), frame_vbox);
+
+	banner = mux_banner_new (_("Devices"));
+	gtk_widget_show (banner);
+	gtk_box_pack_start (GTK_BOX (frame_vbox), banner, FALSE, FALSE, 0);
+
+	/* BluetoothChooser adds its own alignment on the left, so balance it
+	   out... */
+	alignment = gtk_alignment_new (0.0, 0.0, 1.0, 1.0);
+	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 12, 12, 0, 12);
+	gtk_widget_show (alignment);
+	gtk_box_pack_start (GTK_BOX (frame_vbox), alignment, TRUE, TRUE, 0);
 
 	/* Device list */
 	priv->chooser = g_object_new (BLUETOOTH_TYPE_CHOOSER,
@@ -1315,28 +1430,38 @@ create_add_page (MoblinPanel *self)
 	g_signal_connect (cell, "activated", G_CALLBACK (pair_clicked), self);
 
 	gtk_widget_show (priv->chooser);
-	gtk_container_add (GTK_CONTAINER (frame), priv->chooser);
-	gtk_widget_show (frame);
-	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 4);
+	gtk_container_add (GTK_CONTAINER (alignment), priv->chooser);
 
 	/* Back button */
 	back_button = gtk_button_new_with_label (_("Back to devices"));
 	gtk_widget_show (back_button);
 	g_signal_connect (back_button, "clicked",
 			G_CALLBACK (set_device_view), self);
-	gtk_box_pack_start (GTK_BOX (vbox), back_button, FALSE, FALSE, 4);
+	gtk_box_pack_start (GTK_BOX (vbox), back_button, FALSE, FALSE, 0);
 
 	/* Right column */
-	frame = mx_gtk_frame_new ();
+	frame = gtk_frame_new (NULL);
+	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
 	gtk_widget_show (frame);
-	vbox = gtk_vbox_new (FALSE, 4);
+	gtk_box_pack_start (GTK_BOX (page), frame, FALSE, FALSE, 0);
+
+	frame_vbox = gtk_vbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (frame_vbox), 0);
+	gtk_widget_show (frame_vbox);
+	gtk_container_add (GTK_CONTAINER (frame), frame_vbox);
+
+	banner = mux_banner_new (_("Settings"));
+	gtk_widget_show (banner);
+	gtk_box_pack_start (GTK_BOX (frame_vbox), banner, FALSE, FALSE, 0);
+
+	vbox = gtk_vbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
 	gtk_widget_show (vbox);
-	gtk_container_add (GTK_CONTAINER (frame), vbox);
-	gtk_box_pack_start (GTK_BOX (page), frame, FALSE, FALSE, 4);
+	gtk_container_add (GTK_CONTAINER (frame_vbox), vbox);
 
 	hbox = gtk_hbox_new (FALSE, 4);
 	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 4);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
 	/* Filter combo */
 	filter = bluetooth_filter_widget_new ();
@@ -1344,14 +1469,13 @@ create_add_page (MoblinPanel *self)
 	bluetooth_filter_widget_set_title (BLUETOOTH_FILTER_WIDGET (filter), _("Only show:"));
 	bluetooth_filter_widget_bind_filter (BLUETOOTH_FILTER_WIDGET (filter),
 					     BLUETOOTH_CHOOSER (priv->chooser));
-	gtk_box_pack_start (GTK_BOX (vbox), filter, FALSE, FALSE, 4);
+	gtk_box_pack_start (GTK_BOX (vbox), filter, FALSE, FALSE, 0);
 
 	/* Button for PIN options file */
 	pin_button = gtk_button_new_with_label (_("PIN options"));
 	gtk_widget_show (pin_button);
-	g_signal_connect (pin_button, "clicked",
-                    G_CALLBACK (pin_options_button_clicked_cb), self);
-	gtk_box_pack_start (GTK_BOX (vbox), pin_button, FALSE, FALSE, 4);
+	g_signal_connect (pin_button, "clicked", G_CALLBACK (pin_options_button_clicked_cb), self);
+	gtk_box_pack_start (GTK_BOX (vbox), pin_button, FALSE, FALSE, 0);
 
 	return page;
 }
@@ -1361,28 +1485,49 @@ create_devices_page (MoblinPanel *self)
 {
 	MoblinPanelPrivate *priv;
 	GtkWidget *page;
-	GtkWidget *frame_title;
-	GtkWidget *vbox, *hbox;
+	GtkWidget *banner;
+	GtkWidget *alignment, *vbox, *vbox2, *hbox;
 	GtkWidget *frame;
 	GtkWidget *power_label;
 	GtkTreeViewColumn *type_column;
 	GtkCellRenderer *cell;
-	KillswitchState switch_state;
 	GtkWidget *tree_view;
+	GtkSizeGroup *size;
 
 	priv = MOBLIN_PANEL_GET_PRIVATE (self);
 
-	page = gtk_hbox_new (FALSE, 0);
+	page = gtk_hbox_new (FALSE, 8);
+	gtk_container_set_border_width (GTK_CONTAINER (page), 8);
 	gtk_widget_show (page);
-	/* Add child widgetry */
-	vbox = gtk_vbox_new (FALSE, 4);
-	gtk_widget_show (vbox);
-	gtk_box_pack_start (GTK_BOX (page), vbox, TRUE, TRUE, 4);
 
-	frame = mx_gtk_frame_new ();
-	frame_title = gtk_label_new ("");
-	gtk_frame_set_label_widget (GTK_FRAME (frame), frame_title);
-	set_frame_title (GTK_FRAME (frame), _("Devices"));
+	size = gtk_size_group_new (GTK_SIZE_GROUP_VERTICAL);
+
+	frame = gtk_frame_new (NULL);
+	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
+	gtk_widget_show (frame);
+	vbox = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox);
+	gtk_container_add (GTK_CONTAINER (frame), vbox);
+	gtk_box_pack_start (GTK_BOX (page), frame, TRUE, TRUE, 0);
+
+	banner = mux_banner_new (_("Devices"));
+	gtk_widget_show (banner);
+	gtk_size_group_add_widget (size, banner);
+	gtk_box_pack_start (GTK_BOX (vbox), banner, FALSE, FALSE, 0);
+
+	/* Add new button */
+	priv->add_new_button = gtk_button_new_with_label (_("Add a new device"));
+	gtk_widget_show (priv->add_new_button);
+	g_signal_connect (priv->add_new_button, "clicked",
+			G_CALLBACK (set_scanning_view), self);
+	gtk_box_pack_end (GTK_BOX (banner), priv->add_new_button, FALSE, FALSE, 0);
+
+	/* BluetoothChooser adds its own alignment on the left, so balance it
+	   out... */
+	alignment = gtk_alignment_new (0.0, 0.0, 1.0, 1.0);
+	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 12, 12, 0, 12);
+	gtk_widget_show (alignment);
+	gtk_box_pack_start (GTK_BOX (vbox), alignment, TRUE, TRUE, 0);
 
 	/* Device list */
 	priv->display = g_object_new (BLUETOOTH_TYPE_CHOOSER,
@@ -1425,28 +1570,30 @@ create_devices_page (MoblinPanel *self)
 	g_signal_connect (cell, "activated", G_CALLBACK (remove_clicked_cb), self);
 
 	gtk_widget_show (priv->display);
-	gtk_container_add (GTK_CONTAINER (frame), priv->display);
-	gtk_widget_show (frame);
-	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 4);
-
-	/* Add new button */
-	priv->add_new_button = gtk_button_new_with_label (_("Add a new device"));
-	gtk_widget_show (priv->add_new_button);
-	g_signal_connect (priv->add_new_button, "clicked",
-			G_CALLBACK (set_scanning_view), self);
-	gtk_box_pack_start (GTK_BOX (vbox), priv->add_new_button, FALSE, FALSE, 4);
+	gtk_container_add (GTK_CONTAINER (alignment), priv->display);
 
 	/* Right column */
-	frame = mx_gtk_frame_new ();
+	frame = gtk_frame_new (NULL);
+	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
 	gtk_widget_show (frame);
-	vbox = gtk_vbox_new (FALSE, 4);
+	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox);
 	gtk_container_add (GTK_CONTAINER (frame), vbox);
-	gtk_box_pack_start (GTK_BOX (page), frame, FALSE, FALSE, 4);
+	gtk_box_pack_start (GTK_BOX (page), frame, FALSE, FALSE, 0);
+
+	banner = mux_banner_new (_("Settings"));
+	gtk_size_group_add_widget (size, banner);
+	gtk_widget_show (banner);
+	gtk_box_pack_start (GTK_BOX (vbox), banner, FALSE, FALSE, 0);
+
+	vbox2 = gtk_vbox_new (FALSE, 4);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 12);
+	gtk_widget_show (vbox2);
+	gtk_box_pack_start (GTK_BOX (vbox), vbox2, TRUE, TRUE, 0);
 
 	hbox = gtk_hbox_new (FALSE, 4);
 	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 4);
+	gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 4);
 
 	/* Power switch */
 	/* Translators: This string appears next to a toggle switch which controls
@@ -1457,45 +1604,24 @@ create_devices_page (MoblinPanel *self)
 	gtk_box_pack_start (GTK_BOX (hbox), power_label, FALSE, FALSE, 4);
 
 	priv->power_switch = mx_gtk_light_switch_new ();
-	if (priv->killswitch != NULL) {
-		if (bluetooth_killswitch_has_killswitches (priv->killswitch) == FALSE) {
-			gtk_widget_set_sensitive (priv->power_switch, FALSE);
-			gtk_widget_set_sensitive (priv->add_new_button, FALSE);
-		} else {
-			switch_state = bluetooth_killswitch_get_state (priv->killswitch);
-
-			switch (switch_state) {
-				case KILLSWITCH_STATE_UNBLOCKED:
-					mx_gtk_light_switch_set_active
-						(MX_GTK_LIGHT_SWITCH (priv->power_switch),
-						 TRUE);
-					gtk_widget_set_sensitive (priv->add_new_button, TRUE);
-					gtk_widget_set_sensitive (priv->power_switch, TRUE);
-				break;
-				case KILLSWITCH_STATE_SOFT_BLOCKED:
-					mx_gtk_light_switch_set_active
-						(MX_GTK_LIGHT_SWITCH (priv->power_switch),
-							FALSE);
-				break;
-				case KILLSWITCH_STATE_HARD_BLOCKED:
-				case KILLSWITCH_STATE_NO_ADAPTER:
-				default:
-					gtk_widget_set_sensitive (priv->power_switch, FALSE);
-					gtk_widget_set_sensitive (priv->add_new_button, FALSE);
-				break;
-			}
-		}
-	} else {
-		gtk_widget_set_sensitive (priv->power_switch, FALSE);
-		gtk_widget_set_sensitive (priv->add_new_button, FALSE);
-	}
-	g_signal_connect  (priv->killswitch, "state-changed",
-			G_CALLBACK (killswitch_state_changed_cb), self);
+	g_signal_connect  (priv->powerswitch, "state-changed",
+			G_CALLBACK (powerswitch_state_changed_cb), self);
 	g_signal_connect (priv->power_switch, "switch-flipped",
 			G_CALLBACK (power_switch_toggled_cb), self);
 	gtk_widget_show (priv->power_switch);
 	gtk_box_pack_start (GTK_BOX (hbox), priv->power_switch, FALSE, FALSE, 4);
 
+	priv->visible_label = gtk_label_new (NULL);
+	gtk_misc_set_alignment (GTK_MISC (priv->visible_label), 0.0, 0.5);
+	gtk_label_set_line_wrap (GTK_LABEL (priv->visible_label), TRUE);
+	gtk_label_set_width_chars (GTK_LABEL (priv->visible_label), 30);
+	gtk_box_pack_start (GTK_BOX (vbox2), priv->visible_label, FALSE, FALSE, 4);
+
+	priv->visible_button = gtk_button_new_with_label (_("Make visible on Bluetooth"));
+	g_signal_connect (priv->visible_button, "clicked", G_CALLBACK (visible_button_cb), self);
+	gtk_widget_show (priv->visible_button);
+	gtk_box_pack_start (GTK_BOX (vbox2), priv->visible_button, FALSE, FALSE, 4);
+
 	/* Button for Send file */
 	priv->send_button = gtk_button_new_with_label (_("Send file from your computer"));
 	gtk_widget_show (priv->send_button);
@@ -1503,9 +1629,11 @@ create_devices_page (MoblinPanel *self)
                     G_CALLBACK (send_file_button_clicked_cb), self);
 	g_signal_connect (priv->display, "selected-device-changed",
 			G_CALLBACK (selected_device_changed_cb), self);
-	gtk_box_pack_start (GTK_BOX (vbox), priv->send_button, FALSE, FALSE, 4);
+	gtk_box_pack_start (GTK_BOX (vbox2), priv->send_button, FALSE, FALSE, 4);
 
-	enable_send_file (self);
+        powerswitch_state_changed_cb (priv->powerswitch,
+                                      bluetooth_powerswitch_get_state (priv->powerswitch),
+                                      self);
 
 	return page;
 }
@@ -1607,7 +1735,7 @@ moblin_panel_init (MoblinPanel *self)
 
 	bluetooth_agent_setup (priv->agent, AGENT_PATH);
 
-	priv->killswitch = bluetooth_killswitch_new ();
+	priv->powerswitch = bluetooth_powerswitch_new ();
 	bluetooth_plugin_manager_init ();
 
 	gtk_box_set_homogeneous (GTK_BOX (self), FALSE);
@@ -1657,9 +1785,9 @@ moblin_panel_dispose (GObject *object)
 
 	bluetooth_plugin_manager_cleanup ();
 
-	if (priv->killswitch != NULL) {
-		g_object_unref (priv->killswitch);
-		priv->killswitch = NULL;
+	if (priv->powerswitch != NULL) {
+		g_object_unref (priv->powerswitch);
+		priv->powerswitch = NULL;
 	}
 
 	if (priv->agent != NULL) {
@@ -1708,12 +1836,19 @@ moblin_panel_new (void)
 
 /**
  *
- * moblin_panel_reset_view:
+ * moblin_panel_hidden:
  *
  * @self: A #MoblinPanel widget
  **/
 void
-moblin_panel_reset_view (MoblinPanel *self)
+moblin_panel_hidden (MoblinPanel *self)
 {
 	set_current_page (self, PAGE_DEVICES);
 }
+
+void
+moblin_panel_shown (MoblinPanel *self)
+{
+	set_current_page (self, PAGE_DEVICES);
+	make_discoverable (self);
+}
